(* val read_list: ( unit -> 'a option) -> 'a list *)
val fold_map: ('a -> 'b) -> ('b -> 'b -> 'b) -> 'b -> ('a list) -> 'b
val get_max: ('a -> float) -> ('a list) -> ('a * 'a list) (*  pair s.t. first element is maximal and  second element is list without first element *)
val max_el: ('a -> float) -> ('a list) -> int (* first element has index 0 *)
val prod: ('a -> 'b -> 'c) -> ('a list) -> ('b list) -> ('c list)  (* apply #1 to the cartesion product of #2 and #3 *)
val disjoint: 'a list -> 'a list -> bool
val intersection: 'a list -> 'a list -> 'a list
val compose: ('b -> 'c) -> ('a -> 'b) -> ('a -> 'c) (* function composition *)
val int_list: int -> int -> int list (* list of integers from #1 to #2 *)
val count: ('a -> bool) -> ('a list) -> int
val pos: 'a list -> 'a -> int   (* position of first occurence in list; -1 if none *)
val drop_last: 'a list -> int -> 'a list (* #1 without the #2 last elements *)
val drop_nth: 'a list -> int -> 'a list (* #1 without the #2-th element *)
val unique: 'a list -> bool (* true if no element occurs more than once in #1 *)
val diff: 'a list -> 'a list -> 'a list (* #1 - #2 *)
val union: 'a list -> 'a list -> 'a list (* #1 union #2 *)
val find_index: ('a -> bool) -> 'a list -> int list (* list of all indices of elements of #2 that fulfill #1 *)
val find_index2: ('a -> 'b -> bool) -> 'a list -> 'b list -> (int * int) list (* list of all index pairs of elements of #2 and #3 that fulfill #1 *)
val pair: 'a -> 'b -> ('a * 'b) (* the pair (#1, #2) *)
val repeat: 'a -> int -> 'a list (* #2 times #1 *)
val cart_prod: 'a list -> 'b list -> ('a * 'b) list (* the Cartesian product of #1 and #2 *)

val exists_opt: ('a -> 'b option) -> 'a list -> 'b option
val partition_opt: ('a -> 'b option) -> ('a list) -> ('b list * 'a list)
val map_opt: ('a -> 'b) -> 'a option -> 'b option
val filter_opt: 'a option list -> 'a list (* list of all somes *)
val is_something: 'a option -> bool
val de_opt: 'a option -> 'a

val int_exists: int -> int -> (int -> bool) -> int (* if there is an integer in [#1, #2] such that #3 holds, then returns it, otherwise -1 *)
val int_forall: int -> int -> (int -> bool) -> int (* if for all integer in [#1, #2] #3 holds, then returns -1, otherwise counter-example *)

val array_exists : ('a -> bool) -> 'a array -> int (* returns witness, and -1 otherwise *)
val array_exists_i : ('a -> int -> bool) -> 'a array -> int (* returns witness, and -1 otherwise *)
val array_exists2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> int (* returns witness, and -1 otherwise *)
val array_find_index : ('a -> bool) -> 'a array -> int list (* list of all witness indices *)
val array_for_all : ('a -> bool) -> 'a array -> int (* returns witness, and -1 otherwise *)
val array_for_all2 : ('a -> 'b -> bool) -> 'a array -> 'b array -> int (* returns witness, and -1 otherwise *)
val array_for_all_i : ('a -> int -> bool) -> 'a array -> int (* returns witness, and -1 otherwise *)
val array_count: ('a -> bool) -> ('a array) -> int (* number of array elements on that function holds *)
val array_max_el: ('a -> float) -> ('a array) -> (int * float)

val is_a_number: float -> bool

val matrix_init: int -> int -> (int -> int -> 'a) -> 'a array array
val vector_prod: float array -> float array -> float
val vector_sum: float array -> float array -> float array
val vector_diff: float array -> float array -> float array
val vector_scalar_mult: float -> float array -> float array
val transpose: 'a array array -> 'a array array

